#!/bin/bash
# Adds an IP to the iptables drop list (if linux)
# Adds an IP to the ipfilter drop list (if solaris, freebsd or netbsd)
# Adds an IP to the ipsec drop list (if aix)
# Requirements: Linux with iptables, Solaris/FreeBSD/NetBSD with ipfilter or AIX with IPSec
# Expect: srcip
# Author: Ahmet Ozturk (ipfilter and IPSec)
# Author: Daniel B. Cid (iptables)
# Author: cgzones 
# Last modified: Oct 04, 2012

# Load ASL firewall config
if [ -f /var/asl/data/firewall.cnf ]; then
	source /var/asl/data/firewall.cnf
fi

UNAME=`uname`
ECHO="/bin/echo"
GREP="/bin/grep"
IPTABLES=""
IPTABLES_EXISTS=""
IP4TABLES="/sbin/iptables"
IP4TABLES_EXISTS="/sbin/iptables"
if [ "$FW_WAIT_SUPPORT" = "yes" ]; then
	IP4TABLES="/sbin/iptables -w 2 "
fi

IP6TABLES="/sbin/ip6tables"
IP6TABLES_EXISTS="/sbin/ip6tables"

IPFILTER="/sbin/ipf"
if [ "X$UNAME" = "XSunOS" ]; then
    IPFILTER="/usr/sbin/ipf"
fi    
GENFILT="/usr/sbin/genfilt"
LSFILT="/usr/sbin/lsfilt"
MKFILT="/usr/sbin/mkfilt"
RMFILT="/usr/sbin/rmfilt"
ARG1=""
ARG2=""
RULEID=""
ACTION=$1
USER=$2
IP=$3
PWD=`pwd`
LOCK="${PWD}/fw-drop"
LOCK_PID="${PWD}/fw-drop/pid"

COMMENT=no
if [ "$COMMENT" == "yes" ]; then
	if /sbin/lsmod | grep xt_comment &> /dev/null ; then
		COMMENT="-m comment --comment $4"
	fi
else
	COMMENT=
fi



LOCAL=`dirname $0`;
cd $LOCAL
cd ../
filename=$(basename "$0")

LOG_FILE="${PWD}/../logs/active-responses.log"

echo "`date` $0 $1 $2 $3 $4 $5" >> ${LOG_FILE}


# Checking for an IP
if [ "x${IP}" = "x" ]; then
   echo "$0: <action> <username> <ip>" 
   exit 1;
fi

case "${IP}" in
    *:* )
        IPTABLES=$IP6TABLES
        IPTABLES_EXISTS=$IP6TABLES_EXISTS
        ;;
    *.* )
        IPTABLES=$IP4TABLES
        IPTABLES_EXISTS=$IP4TABLES_EXISTS
        ;;
    * ) echo "`date` Unable to run active response (invalid IP: '${IP}')." >> ${LOG_FILE} && exit 1;;
esac

# This number should be more than enough (even if a hundred
# instances of this script is ran together). If you have
# a really loaded env, you can increase it to 75 or 100.
MAX_ITERATION="200"

# Lock function
lock()
{
    i=0;

	# ASL Firewall lock check
	y=0;
        while [ -f /var/lock/subsys/asl-firewall.loading ]; do
                if [ $y -gt 20 ]; then
			# Log that it failed here
                        break
                fi
                y=$(( $y + 1 ))
                echo FOUND: $y
                sleep 1

        done



    # Providing a lock.
    while [ 1 ]; do
        mkdir ${LOCK} > /dev/null 2>&1
        MSL=$?
        if [ "${MSL}" = "0" ]; then
            # Lock aquired (setting the pid)
            echo "$$" > ${LOCK_PID}
            return;
        fi

        # Getting currently/saved PID locking the file
        C_PID=`cat ${LOCK_PID} 2>/dev/null`
        if [ "x" = "x${S_PID}" ]; then
            S_PID=${C_PID}
        fi

        # Breaking out of the loop after X attempts
        if [ "x${C_PID}" = "x${S_PID}" ]; then
            i=`expr $i + 1`;
        fi

        # Sleep 1 after 10/25 interactions
        if [ "$i" = "10" -o "$i" = "25" ]; then
            sleep 1;
        fi

        i=`expr $i + 1`;

        # So i increments 2 by 2 if the pid does not change.
        # If the pid keeps changing, we will increments one
        # by one and fail after MAX_ITERACTION

        if [ "$i" = "${MAX_ITERATION}" ]; then
            kill="false"
            for pid in `pgrep -f "${filename}"`; do
                if [ "x${pid}" = "x${C_PID}" ]; then
                    # Unlocking and exiting
                    kill -9 ${C_PID}
                    echo "`date` Killed process ${C_PID} holding lock." >> ${LOG_FILE}
                    kill="true"
                    unlock;
                    i=0;
                    S_PID="";
                    break;
                fi
            done

            if [ "x${kill}" = "xfalse" ]; then
                echo "`date` Unable kill process ${C_PID} holding lock." >> ${LOG_FILE}
                # Unlocking and exiting
                unlock;
                exit 1;
            fi
        fi
    done
}

# Unlock function
unlock()
{
   rm -rf ${LOCK} 
}



# Blocking IP
if [ "x${ACTION}" != "xadd" -a "x${ACTION}" != "xdelete" ]; then
   echo "$0: invalid action: ${ACTION}"
   exit 1;
fi



# We should run on linux
if [ "X${UNAME}" = "XLinux" ]; then
   if ! /sbin/iptables-save | grep ASL-ACTIVE-RESPONSE 2>/dev/null | grep -q DROP ; then
     	${IPTABLES} -N ASL-ACTIVE-RESPONSE >/dev/null
     	${IPTABLES} -N ASL-ACTIVE-RESPONSE-LOG >/dev/null
     	${IPTABLES} -I INPUT -j  ASL-ACTIVE-RESPONSE >/dev/null
     	${IPTABLES} -A ASL-ACTIVE-RESPONSE-LOG -m limit --limit 10/minute --limit-burst 2 -j LOG  --log-level info --log-prefix "ASL_AR_DROP "  --log-tcp-sequence  --log-tcp-options  --log-ip-options >/dev/null
     	${IPTABLES} -A ASL-ACTIVE-RESPONSE-LOG -j DROP >/dev/null
   fi


  
   if [ "x${ACTION}" = "xadd" ]; then
	#Check to make sure its an actual IP, dont shun FQDNs
	echo "${IP}" | egrep "^([0-9]{1,3})[.]([0-9]{1,3})[.]([0-9]{1,3})[.]([0-9]{1,3})$" > /dev/null 2>&1
	if [ ! $? = 0 ]; then
    		echo "`date` FQDN hostname without IP not allowed: ${IP}" >> ${PWD}/../logs/active-responses.log
        exit 1;
	fi

	#Dont shun broadcasts
	echo "${IP}" | egrep "^(0.0.0.0|255.255.255.255)$" > /dev/null 2>&1
	if [ $? = 0 ]; then
    		echo "`date` Broadcast address not allowed: ${IP}" >> ${PWD}/../logs/active-responses.log
        	exit 1;
	fi

	#check to make sure its not loopback
	echo "${IP}" | egrep "^127." > /dev/null 2>&1
	if [ $? = 0 ]; then
    		echo "`date` Use of 127.0.0.0/8 IP address not allowed: ${IP}" >> ${PWD}/../logs/active-responses.log
        exit 1;
        fi

	ARG1="-I ASL-ACTIVE-RESPONSE -s ${IP} -j ASL-ACTIVE-RESPONSE-LOG $COMMENT "
      	ARG2="-I FORWARD -s ${IP} -j ASL-ACTIVE-RESPONSE-LOG $COMMENT "
   else
      ARG1="-D ASL-ACTIVE-RESPONSE -s ${IP} -j ASL-ACTIVE-RESPONSE-LOG $COMMENT"
      ARG2="-D FORWARD -s ${IP} -j  ASL-ACTIVE-RESPONSE-LOG $COMMENT"
   fi
   
   # Checking if iptables is present
   #if [ ! -x ${IPTABLES} ]; then
   if [ ! -x ${IPTABLES_EXISTS} ]; then
      #IPTABLES="/usr"${IPTABLES}
      IPTABLES_EXISTS="/usr"${IPTABLES_EXISTS}
      if [ ! -x ${IPTABLES} ]; then
        echo "$0: can not find iptables"
        exit 0;
      fi
   fi

   # Executing and exiting
   COUNT=0;
   lock;	
   if [ "x${ACTION}" = "xadd" ]; then
	   while [ 1 ]; do
		${IPTABLES} ${ARG1} 
		RES=$?
		if [ $RES = 0 ]; then
		    break;
		else
		    COUNT=`expr $COUNT + 1`;
		    echo "`date` Unable to run (iptables returning != $RES): $COUNT - $0 $1 $2 $3 $4 $5" >> ${LOG_FILE}
		    sleep $COUNT;

		    if [ $COUNT -gt 4 ]; then
			break;
		    fi    
		fi
	   done
   else 
	while [ 1 ]; do
		${IPTABLES} ${ARG1}  >/dev/null 2>&1
		RES=$?
		if [ $RES = 1 ]; then
			break;
		else
		    COUNT=`expr $COUNT + 1`;
		    if [ $COUNT -gt 30 ]; then
			break;
		    fi    
		fi
	
	done
   fi
   
   COUNT=0;
   if [ "x${ACTION}" = "xadd" ]; then
	   while [ 1 ]; do
		${IPTABLES} ${ARG2} 
		RES=$?
		if [ $RES = 0 ]; then
		    break;
		else
		    COUNT=`expr $COUNT + 1`;
		    echo "`date` Unable to run (iptables returning != $RES): $COUNT - $0 $1 $2 $3 $4 $5" >> ${LOG_FILE}
		    sleep $COUNT;

		    if [ $COUNT -gt 4 ]; then
			break;
		    fi       
		fi
	   done
   else
        while [ 1 ]; do
                ${IPTABLES} ${ARG2}  >/dev/null 2>&1
                RES=$?
                if [ $RES = 1 ]; then
                        break;
                else
                    COUNT=`expr $COUNT + 1`;
                    if [ $COUNT -gt 30 ]; then
                        break;
                    fi
                fi

        done

   fi

   unlock;
            
   exit 0;
   
# FreeBSD, SunOS or NetBSD with ipfilter
elif [ "X${UNAME}" = "XFreeBSD" -o "X${UNAME}" = "XSunOS" -o "X${UNAME}" = "XNetBSD" ]; then
   
   # Checking if ipfilter is present
   ls ${IPFILTER} >> /dev/null 2>&1
   if [ $? != 0 ]; then
      exit 0;
   fi    

   # Checking if echo is present
   ls ${ECHO} >> /dev/null 2>&1
   if [ $? != 0 ]; then
       exit 0;
   fi    
   
   if [ "x${ACTION}" = "xadd" ]; then
      ARG1="\"@1 block out quick from any to ${IP}\""
      ARG2="\"@1 block in quick from ${IP} to any\""
      IPFARG="${IPFILTER} -f -"
   else
      ARG1="\"@1 block out quick from any to ${IP}\""
      ARG2="\"@1 block in quick from ${IP} to any\""
      IPFARG="${IPFILTER} -rf -"
   fi
  
   # Executing it 
   eval ${ECHO} ${ARG1}| ${IPFARG}       
   eval ${ECHO} ${ARG2}| ${IPFARG}
   
   exit 0;

# AIX with ipsec
elif [ "X${UNAME}" = "XAIX" ]; then

  # Checking if genfilt is present
  ls ${GENFILT} >> /dev/null 2>&1
  if [ $? != 0 ]; then
     exit 0;
  fi
         
  # Checking if lsfilt is present
  ls ${LSFILT} >> /dev/null 2>&1
  if [ $? != 0 ]; then
     exit 0;
  fi
  # Checking if mkfilt is present
  ls ${MKFILT} >> /dev/null 2>&1
  if [ $? != 0 ]; then
     exit 0;
  fi
         
  # Checking if rmfilt is present
  ls ${RMFILT} >> /dev/null 2>&1
  if [ $? != 0 ]; then
     exit 0;
  fi

  if [ "x${ACTION}" = "xadd" ]; then
    ARG1=" -v 4 -a D -s ${IP} -m 255.255.255.255 -d 0.0.0.0 -M 0.0.0.0 -w B -D \"Access Denied by OSSEC-HIDS\"" 
    #Add filter to rule table
    eval ${GENFILT} ${ARG1}
    
    #Deactivate  and activate the filter rules.
    eval ${MKFILT} -v 4 -d
    eval ${MKFILT} -v 4 -u
  else
    # removing a specific rule is not so easy :(
     eval ${LSFILT} -v 4 -O  | ${GREP} ${IP} | 
     while read -r LINE
     do
         RULEID=`${ECHO} ${LINE} | cut -f 1 -d "|"`
         let RULEID=${RULEID}+1
         ARG1=" -v 4 -n ${RULEID}"
         eval ${RMFILT} ${ARG1}
     done
    #Deactivate  and activate the filter rules.
    eval ${MKFILT} -v 4 -d
    eval ${MKFILT} -v 4 -u
  fi

else
    exit 0;
fi
